/************************************************
 * Filename: mastermind.hpp
 * Summary:  Agenda VR3 FLTK MasterMind game
 * Created:  24 February 2001
 * Author:   Kent Dahl ( MenThal@bigfoot.com )
 * Description: A silly little buggy logic game
 ************************************************/


#ifndef DEBUG
#define DEBUG 0
#endif



#include <vector>


//#define Value int

typedef int Value;


class ValueListener {
public:
    virtual void valueUpdated( Value a_newValue ) = 0;
};


class DigitConstraints  {
public:
    /**
     * Value limits
     */ 
    const Value m_min, m_max;

    /**
     * Should we "halt" at the extreme values or wraparound?
     */ 
    const bool m_wrapAround;


    DigitConstraints( Value a_max, bool a_wrap = false ) ;

    virtual Value succ( Value a_current ) const;
    virtual Value pred( Value a_current ) const;

    virtual Value randomValue() const;
    virtual bool isValid( Value val ) const;

};

class Digit {
protected:
    Value value;
    const DigitConstraints &m_constraints;
    ValueListener *m_pListener;
public:
    Digit( Value val, 
           const DigitConstraints &a_constraints,
           ValueListener *a_pListener = 0) ;

    void  setValue( Value val );

    Value getValue() const;


    virtual void update();
    virtual bool equals( const Digit& a_other ) const;

    virtual void increase();

    virtual void decrease();

    virtual bool isValid();

    virtual void randomize();



};




class CodeConstraints : public DigitConstraints{
public:
    const unsigned int m_numberOfDigits;
    const bool         m_uniqueDigits;
    
    CodeConstraints( Value a_max, 
                     int a_numDigits = 4,
                     bool a_unique = true, 
                     bool a_wrap = false ) 
        : DigitConstraints( a_max, a_wrap ),
          m_numberOfDigits( a_numDigits ),
          m_uniqueDigits( a_unique )

        { }
};




/** 
 * A Code entry
 */
class Code {
protected:
    typedef std::vector<Digit*> DigitList;

    const CodeConstraints &m_constraints;
    DigitList m_digits;

    void deleteDigits();

public:
    Code( const CodeConstraints &constraints );
    virtual ~Code();



    virtual bool isValid() const ;

    virtual bool equals( const Code& a_other ) ;

    virtual int  numberOfEqual( const Code& a_other );

    /** returns the number of existings values in the entry that
     * do appear in a_other, but not on the correct place.
     * Is only guaranteed to work if both this and the other Code
     * object isValid()
     * (duplicate variant not yet working)
     */
    virtual int  numberOfExists( const Code& a_other );
    virtual int  numberOfExists_unique( const Code& a_other );
    virtual int  numberOfExists_duplicate( const Code& a_other );


    virtual void randomize();


    virtual void print(ostream& out);

    virtual void add( Digit* a_digit );



};


class CodeEntryResult {
public:
    const int  m_numCorrect;
    const int  m_numExists;

    CodeEntryResult(int a_correct, int a_exists )
        : m_numCorrect( a_correct ),
          m_numExists(  a_exists  )
        {
        }
};





class ICodeEntryListener {
public:
    virtual ~ICodeEntryListener(){}
    virtual void gameBegins() = 0;
    virtual void playerWon( int a_numTries ) = 0;
    virtual void entered( CodeEntryResult ) = 0;
    virtual void gameOver() = 0;
    virtual void invalidEntry() = 0;

};




class CodeEntryListener_Base : public ICodeEntryListener {
public:
    virtual void gameBegins(){};
    virtual void playerWon( int a_numTries ){};
    virtual void entered( CodeEntryResult ){};
    virtual void gameOver(){};
    virtual void invalidEntry(){};
};




class GameConfig : public CodeConstraints {
public:
    const int m_maxTries;
    GameConfig( int a_maxTries = 10, 
                int a_numDigits = 4, 
                int a_numValues = 6,
                bool a_uniqueDigits = true 
                ) 
        : CodeConstraints( a_numValues-1, a_numDigits, a_uniqueDigits ),
          m_maxTries( a_maxTries ) { }
};




class Game {
    Code m_answer;
    ICodeEntryListener &m_codeEntryListener;
    int m_numTries;
    bool m_gameover;
public:
    const GameConfig &m_config;
    
    
    Game( const GameConfig   &a_config,
          ICodeEntryListener &a_listener )        ;
    virtual ~Game();
    virtual void enter( const Code &a_entry );
    virtual void restart();

    void print( ostream& out ){
        m_answer.print( out );
        out << ", tries:"<<m_numTries<<"\n";
    }
    
};




////////////////////////////
