#include "mastermind.hpp"

#include <vector>

/**
 * Using CppUnit, the C++ unit testing framework; 
 * see: https://sourceforge.net/projects/cppunit
 */

#include <cppunit/TestCase.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TextTestResult.h>
#include <cppunit/TestSuite.h>
#include <cppunit/TestRegistry.h>


using namespace CppUnit;

class MyMMTest : public TestCase {
    GameConfig *pGameConfig, *pDupGameConfig;
    Code *codes;
    Code *dupCodes;
    static int NUM_CODES;
    static int CODE_ARRAY[][4];
public:
    MyMMTest( ) : TestCase() {}

    void setUp()
        {
            pGameConfig    = new GameConfig();
            pDupGameConfig = new GameConfig(10,4,6,false);
            codes    = new Code[NUM_CODES]( *pGameConfig );
            dupCodes = new Code[NUM_CODES]( *pDupGameConfig );


            for( int i = 0; i < NUM_CODES  ;i++)
                for( int j = 0; j<4;j++)
                {
                    codes[i].add( new Digit(CODE_ARRAY[i][j], *pGameConfig) );
                    dupCodes[i].add( new Digit(CODE_ARRAY[i][j], 
                                               *pDupGameConfig) );
                }
        }
    void tearDown()
        {
            delete pGameConfig;
            delete pDupGameConfig;
            delete [] codes;
            delete [] dupCodes;
        }

    void testEqual(){
        assertImplementation( !codes[0].equals(codes[1]) ,"equals");
        assertImplementation( codes[1].equals(codes[2]) ,"equals");
    }
    void testValid(){
        assertImplementation( !codes[0].isValid(), "valid",__LINE__,__FILE__); 
        assertImplementation( codes[1].isValid(), "valid",__LINE__,__FILE__); 
        assertImplementation( codes[2].isValid(), "valid",__LINE__,__FILE__); 
        assertImplementation( codes[3].isValid(), "valid",__LINE__,__FILE__); 
        assertImplementation( !codes[4].isValid(), "valid",__LINE__,__FILE__); 
        assertImplementation( !codes[5].isValid(), "valid",__LINE__,__FILE__); 
    }
    void testNumEqual(){
        assertEquals( 1, codes[0].numberOfEqual(codes[1]), __LINE__,__FILE__);
        assertEquals( 4, codes[1].numberOfEqual(codes[2]), __LINE__,__FILE__);
        assertEquals( 4, codes[0].numberOfEqual(codes[0]), __LINE__,__FILE__);
        assertEquals( 0, codes[2].numberOfEqual(codes[3]), __LINE__,__FILE__);
    }
    void testNumExists(){
        assertEquals( 0, codes[0].numberOfExists(codes[1]), __LINE__,__FILE__);
        assertEquals( 0, codes[1].numberOfExists(codes[2]), __LINE__,__FILE__);

        //test duplicate
        assertEquals( 0, dupCodes[0].numberOfExists(dupCodes[1]),
                      __LINE__,__FILE__);

        assertEquals( 0, dupCodes[1].numberOfExists(dupCodes[2]),
                      __LINE__,__FILE__);
        assertEquals( 4, dupCodes[1].numberOfExists(dupCodes[3]),
                      __LINE__,__FILE__);

        assertEquals( 2, dupCodes[5].numberOfExists(dupCodes[6]),
                      __LINE__,__FILE__);




    }
    
    void testNumEqualsAndExistsSymmetry(){

        for(int i = 0;i<NUM_CODES;i++)
            if ( codes[i].isValid() )
                for(int j = 0;j<NUM_CODES;j++){
                    
                    // test unique
                    if( codes[j].isValid() )
                        assertImplementation( 
                            4 >= codes[i].numberOfExists(codes[j]) 
                            + codes[i].numberOfEqual(codes[j]),
                            " uniq ", __LINE__,__FILE__);
                }


      for(int i = 0;i<NUM_CODES;i++)
            if ( dupCodes[i].isValid() )
                for(int j = 0;j<NUM_CODES;j++){
                    
                    // test duplicate
                    if( dupCodes[j].isValid() )
                    {
                        // with duplicate entries
                        assertImplementation( 
                            4 >= dupCodes[i].numberOfExists(dupCodes[j]) 
                            + dupCodes[i].numberOfEqual(dupCodes[j]),
                            " dup ", __LINE__,__FILE__);
                    }

                }







                
    }


    virtual void registerTests(TestSuite *suite)
        {
            suite->addTest(
                new TestCaller<MyMMTest>("equality",
                                         &MyMMTest::testEqual));
            suite->addTest(
                new TestCaller<MyMMTest>("validity",
                                         &MyMMTest::testValid));
             suite->addTest(
                new TestCaller<MyMMTest>("number of equal digits",
                                         &MyMMTest::testNumEqual));
             suite->addTest(
                 new TestCaller<MyMMTest>("number of existing digits",
                                          &MyMMTest::testNumExists));
             suite->addTest(
                 new TestCaller<MyMMTest>("number of ... digits symmetry",
                                          &MyMMTest::testNumEqualsAndExistsSymmetry));

        }

};

int MyMMTest::NUM_CODES = 10;
int MyMMTest::CODE_ARRAY[][4] = { {0,0,0,0}, //0 
                                  {0,1,2,3},
                                  {0,1,2,3},
                                  {3,2,1,0}, 
                                  {0,0,1,2}, //4
                                  {1,2,3,1},
                                  {1,1,2,1},
                                  {0,0,0,0},
                                  {0,0,0,0},
                                  {0,0,0,0}, //9
                                  {0,0,0,0},
                                  {0,0,0,0}, //
                                  {0,0,0,0} };


MyMMTest mmtest;


int main()
{

    TextTestResult res;
    TestSuite suite;

    mmtest.registerTests(&suite);
    
    std::vector<CppUnit::Test*> tests = 
        CppUnit::TestRegistry::getRegistry().getAllTests();
    
    for(std::vector<CppUnit::Test*>::iterator it=tests.begin();
        it!=tests.end(); ++it) {
        
        (*it)->run(&res);
        
    }
    std::cout << res << endl;

 
}
